#!/usr/bin/env python
# Use "env" to ensure that the Python interpreter used is the first in your $PATH

# Import useful modules
import numpy as np
import matplotlib.pyplot as plt
from scipy import stats

# Generate linearly spaced x values 
xmin=0.
xmax=10.
dx=1.0
npts=int((xmax-xmin)/dx+1)
x=np.linspace(xmin, xmax, npts)

# Generate a line 
# If you like, try defining a function to do this
slope=4.
intercept=10.
y=slope*x+intercept
plt.plot(x,y,'-',label='Original: y={0}x+{1}'.format(slope,intercept))

# Add Gaussian random noise 
mean=0.
stdev=5.
noise=np.random.normal(mean,stdev,npts)
yn=y+noise
plt.plot(x,yn,'o')

# Fit a line using stats.linregress package from scipy
slope,intercept,r_value,p_value,std_err=stats.linregress(x,yn)
print 'Slope, intercept from scipy are:',slope, intercept
#y=slope*x+intercept
#plt.plot(x,y,'-',label='Best-fit to noise')

# Least-squares linear fit:
# Define G matrix
G=np.zeros([npts,2])
for i in range(npts):
    G[i][0]=x[i]
    G[i][1]=1
# Solve the inverse problem
# generalized inverse would be np.dot(np.linalg.inv(np.dot(G.T,G)),G.T)
slope, intercept = np.dot(np.dot(np.linalg.inv(np.dot(G.T,G)),G.T),yn)
print 'Slope, intercept are:',slope, intercept

# Calculate best-fit line and plot
y=slope*x+intercept
plt.plot(x,y,'-',label='Best fit: y={0:6.3f}x+{1:6.3f}'.format(slope,intercept))

# Plot points and best-fit line
plt.legend(loc='upper left')
plt.show()
